import decimal
from zope.catalog.interfaces import ICatalog
from zope.component import queryUtility
from zope.component.hooks import setSite
from waeup.ikoba.app import Company
from waeup.ikoba.payments.payment import Payment, PaymentItem
from waeup.ikoba.payments.testing import FakePayer, FakePayable
from waeup.ikoba.testing import FunctionalLayer, FunctionalTestCase
from waeup.ikoba.payments.catalog import search


class FunctionalPaymentCatalogTests(FunctionalTestCase):

    layer = FunctionalLayer

    def create_app(self):
        self.root = self.getRootFolder()
        self.app = Company()
        self.root['app'] = self.app
        setSite(self.app)
        return self.app

    def test_catalog_available(self):
        # the payments catalog will be available in ICompany objects
        self.create_app()
        catalog = queryUtility(ICatalog, name='payments_catalog')
        assert catalog is not None

    def test_catalog_catalogs_ipayments(self):
        # the catalog will register IPayment objects stored
        app = self.create_app()
        payment = Payment(payer=FakePayer(), payable=FakePayable())
        app['p1'] = payment
        result = search(query=payment.payment_id, searchtype='payment_id')
        result = [x for x in result]
        self.assertEqual(len(result), 1)

    def test_search_for_payer_id(self):
        # the catalog will search in payer_id field
        app = self.create_app()
        payer = FakePayer()
        payer.payer_id = u'ThePayerID'
        payment = Payment(payer=payer, payable=FakePayable())
        app['p1'] = payment
        result = search(query='ThePayerID', searchtype='payer_id')
        result = [x for x in result]
        self.assertEqual(len(result), 1)

    def test_search_payer_id_starts_with_query(self):
        # we can find payer_ids starting with query string.
        app = self.create_app()
        payer1, payer2, payer3 = FakePayer(), FakePayer(), FakePayer()
        payer1.payer_id = u'PAYER_11'
        payer2.payer_id = u'PAYER_12'
        payer3.payer_id = u'PAYER_20'
        payment1 = Payment(payer=payer1, payable=FakePayable())
        payment2 = Payment(payer=payer2, payable=FakePayable())
        payment3 = Payment(payer=payer3, payable=FakePayable())
        app['p1'], app['p2'], app['p3'] = payment1, payment2, payment3
        result1 = search(query='PAYER_1', searchtype='payer_id')
        result1 = [x for x in result1]
        self.assertEqual(len(result1), 2)
        result2 = search(query='PAYER_2', searchtype='payer_id')
        result2 = [x for x in result2]
        self.assertEqual(len(result2), 1)
        result3 = search(query='PAYER_3', searchtype='payer_id')
        result3 = [x for x in result3]
        self.assertEqual(len(result3), 0)
        result4 = search(query='PAYER_11', searchtype='payer_id')
        result4 = [x for x in result4]
        self.assertEqual(len(result4), 1)

    def test_search_amount(self):
        # we can find given amounts starting with query string.
        app = self.create_app()
        payable1, payable2 = FakePayable(), FakePayable()
        payable1.payment_items = (
            PaymentItem(u'Item 1', decimal.Decimal("1.0")), )
        payable2.payment_items = (
            PaymentItem(u'Item 2', decimal.Decimal("2.0")), )
        payment1 = Payment(payer=FakePayer(), payable=payable1)
        payment2 = Payment(payer=FakePayer(), payable=payable2)
        app['p1'], app['p2'] = payment1, payment2
        result1 = search(query=decimal.Decimal("1.00"), searchtype='amount')
        result1 = [x for x in result1]
        self.assertEqual(len(result1), 1)
        result2 = search(query=decimal.Decimal("2.00"), searchtype='amount')
        result2 = [x for x in result2]
        self.assertEqual(len(result2), 1)
        result3 = search(query=decimal.Decimal("3.00"), searchtype='amount')
        result3 = [x for x in result3]
        self.assertEqual(len(result3), 0)
